using System;
using System.Collections;
using System.Threading;
using System.Net;
using System.Net.Sockets;
using System.Text;

/// <summary>
/// Class demonstrates use of multi-threaded socket servers.
/// <author>Tim Lindquist ECET ASU East</author>
/// <version>June 15, 2002</version>
/// </summary>
/// <remarks>
/// This server is multi-threaded in that it creates a new thread to handle
/// each incoming client connection. Note that .NET supports this directly,
/// if you use the asynchronous server sockets (see Using an Asynchronous
/// Server Socket in the .NET Framework Developer's Guide).
/// Each incoming connection causes the server to allocate a new
/// thread object. The communication protocol in this example is simple.
/// The client reads a string from standard input to send to the server.
/// It then reads a string from the server and displays the string
/// on standard output.
/// </remarks>
public class ThreadedServer 
{

	private TcpClient threadClient;
	// the connection number of this client
	private int connNo;

	public ThreadedServer(TcpClient client, int connNo)
	{
		this.threadClient = client;
		this.connNo = connNo;
	}

	/// <summary>
	/// This method is started in a new thread to service a client
	/// request.
	/// </summary>
	public void handleConnect()
	{
		string msg = "Hello Client, you are client number: "+connNo;
		byte[] msgBytes = System.Text.Encoding.ASCII.GetBytes(msg);
		byte[] readBytes = new byte[1024];
		// get the network stream associated with this client. The network
		// stream can be used for both receiving data from and sending data
		// to the client and can be used either synchronously or asynchronously
		NetworkStream ns = threadClient.GetStream();
		// Show the methods to read and write the stream directly rather than
		// creating a StreamReader and StreamWriter as in the client.
		// Using this method, must read the correct number of bytes for the
		// appropriate type and do the conversion from byte array to type.
		// Client is sending a string, so read it all (less than 1025 bytes).
		int howMany = ns.Read(readBytes, 0, readBytes.Length);
		ASCIIEncoding AE = new ASCIIEncoding();
		char[] charArray = AE.GetChars(readBytes, 0, howMany);
		String myString = new String(charArray,0,charArray.Length);
		Console.WriteLine("Received from client {1} the string: {0}",
			myString, connNo);
		ns.Write(msgBytes, 0, msgBytes.Length);
		ns.Flush(); //doesn't seem necessary since its next closed.
		ns.Close();
		threadClient.Close();
	}

	public static void Main (string[] args) 
	{
		// count connections
		int connCount = 0;
		TcpListener tcpl = new TcpListener(9090);
		tcpl.Start();
		Console.WriteLine("ThreadedServer waiting for connections on 9090");
		while (true)
		{
			// accept a connection. Note connection is completed before Accept
			// method returns. The alternative is an asynchronous accept.
			TcpClient tcpClient = tcpl.AcceptTcpClient();
			// create an object to start a thread in to handle the connection
			ThreadedServer tcs = new ThreadedServer(tcpClient, connCount++);
			Thread cThread = new Thread(new ThreadStart(tcs.handleConnect));
			cThread.Start();
		}
		//tcpl.Stop(); not necessary since the loop doesn't have an exit
	}
}
